

def filterbyname(list, name):
    """Find an element with it's name

    Args:
        list (list): List of elements
        name (str): Element name to look for

    Returns:
        int: Id of element if found, Raises Exception otherwise
    """
    for el in list['results']:
        if el['name'] == name:
            return el['id']
    else:
        raise Exception



#!/usr/bin/python
# -*- coding: utf8 -*-


import sys
import log
from base import ForemanBase
from pprint import pprint
from foreman.client import Foreman, ForemanException
from voluptuous import MultipleInvalid

from helper import filterbyname

class ForemanImport(ForemanBase):

    def process_config_arch(self):
        log.log(log.LOG_INFO, "Processing Architectures")
        for arch in self.get_config_section('architecture'):
            try:
                self.validator.arch(arch)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Architecture '{0}': YAML validation Error: {1}".format(arch['name'], e))
                continue

            try:
                arch_id = self.fm.architectures.show(arch['name'])['id']
                log.log(log.LOG_DEBUG, "Architecture '{0}' (id={1}) already present.".format(arch['name'], arch_id))
            except:
                log.log(log.LOG_INFO, "Create Architecture '{0}'".format(arch['name']))
                self.fm.architectures.create( architecture = { 'name': arch['name'] } )



    def process_config_domain(self):
        log.log(log.LOG_INFO, "Processing Domains")
        for domain in self.get_config_section('domain'):
            try:
                self.validator.domain(domain)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Domain '{0}': YAML validation Error: {1}".format(domain['name'], e))
                continue
            try:
                dom_id = self.fm.domains.show(domain['name'])['id']
                log.log(log.LOG_DEBUG, "Domain '{0}' (id={1}) already present.".format(domain['name'], dom_id))
            except:
                dns_proxy_id = False
                try:
                    dns_proxy_id = self.fm.smart_proxies.show(domain['dns-proxy'])['id']
                except:
                    log.log(log.LOG_WARN, "Cannot get ID of DNS Smart Proxy '{0}', skipping".format(domain['dns-proxy']))

                log.log(log.LOG_INFO, "Create Domain '{0}'".format(domain['name']))
                dom_params = []
                if (domain['parameters']):
                    for name,value in domain['parameters'].iteritems():
                        p = {
                            'name':     name,
                            'value':    value
                        }
                        dom_params.append(p)
                dom_tpl = {
                    'name': domain['name'],
                    'fullname': domain['fullname'],
                }
                fixdom = {
                    'domain_parameters_attributes': dom_params
                }

                if dns_proxy_id: dom_tpl['dns_id'] = dns_proxy_id

                domo = self.fm.domains.create( domain = dom_tpl )
                if dom_params:
                    self.fm.domains.update(fixdom, domo['id'])



    def process_config_enviroment(self):
        log.log(log.LOG_INFO, "Processing Environments")
        envlist = self.fm.environments.index(per_page=99999)['results']
        for env in self.get_config_section('environment'):
            try:
                self.validator.enviroment(env)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Environment '{0}': YAML validation Error: {1}".format(env['name'], e))
                continue

            env_id = False
            # fm.media.show(name) does not work, we need to iterate over fm.media.index()
            for envc in envlist:
                if (env['name'] == envc['name']):
                    env_id = envc['id']
                    log.log(log.LOG_DEBUG, "Environment '{0}' (id={1}) already present.".format(env['name'], env_id))
                    continue
            if not env_id:
                log.log(log.LOG_INFO, "Create Environment '{0}'".format(env['name']))
                self.fm.environments.create( environment = { 'name': env['name'] } )



    def process_config_model(self):
        log.log(log.LOG_INFO, "Processing Models")
        for model in self.get_config_section('model'):
            try:
                self.validator.model(model)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Model '{0}': YAML validation Error: {1}".format(model['name'], e))
                continue
            try:
                model_id = self.fm.models.show(model['name'])['id']
                log.log(log.LOG_DEBUG, "Model '{0}' (id={1}) already present.".format(model['name'], model_id))
            except:
                log.log(log.LOG_INFO, "Create Model '{0}'".format(model['name']))
                model_tpl = {
                    'name':             model['name'],
                    'info':             model['info'],
                    'vendor_class':     model['vendor-class'],
                    'hardware_model':   model['hardware-model']
                }
                self.fm.models.create( model = model_tpl )



    def process_config_medium(self):
        log.log(log.LOG_INFO, "Processing Media")
        medialist = self.fm.media.index(per_page=99999)['results']
        for medium in self.get_config_section('medium'):
            try:
                self.validator.medium(medium)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Media '{0}': YAML validation Error: {1}".format(medium['name'], e))
                continue

            medium_id = False
            # fm.media.show(name) does not work, we need to iterate over fm.media.index()
            for mediac in medialist:
                if (mediac['name'] == medium['name']):
                    medium_id = mediac['id']
                    log.log(log.LOG_DEBUG, "Medium '{0}' (id={1}) already present.".format(medium['name'], medium_id))
            if not medium_id:
                log.log(log.LOG_INFO, "Create Medium '{0}'".format(medium['name']))
                medium_tpl = {
                    'name':        medium['name'],
                    'path':        medium['path'],
                    'os_family':   medium['os-family']
                }
                self.fm.media.create( medium = medium_tpl )



    def process_config_settings(self):
        log.log(log.LOG_INFO, "Processing Foreman Settings")
        for setting in self.get_config_section('setting'):
            try:
                self.validator.setting(setting)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot update Setting '{0}': YAML validation Error: {1}".format(setting['name'], e))
                continue

            setting_id = False
            try:
                setting_id = self.fm.settings.show(setting['name'])['id']
            except:
                log.log(log.LOG_WARN, "Cannot get ID of Setting '{0}', skipping".format(setting['name']))

            setting_tpl = {
                'value':            setting['value']
            }

            if setting_id:
                log.log(log.LOG_INFO, "Update Setting '{0}'".format(setting['name']))
                self.fm.settings.update(setting_tpl, setting_id)



    def process_config_smartproxy(self):
        log.log(log.LOG_INFO, "Processing Smart Proxies")
        for proxy in self.get_config_section('smart-proxy'):
            try:
                proxy_id = self.fm.smart_proxies.show(proxy['name'])['id']
                log.log(log.LOG_DEBUG, "Proxy '{0}' (id={1}) already present.".format(proxy['name'], proxy_id))
            except:
                log.log(log.LOG_INFO, "Create Smart Proxy '{0}'".format(proxy['name']))
                proxy_tpl = {
                    'name': proxy['name'],
                    'url': proxy['url'],
                }
                try:
                    self.fm.smart_proxies.create( smart_proxy = proxy_tpl )
                except:
                    log.log(log.LOG_WARN, "Cannot create Smart Proxy '{0}'. Is the Proxy online? ".format(proxy['name']))



    def process_config_subnet(self):
        log.log(log.LOG_INFO, "Processing Subnets")
        for subnet in self.get_config_section('subnet'):
            try:
                self.validator.subnet(subnet)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Subnet '{0}': YAML validation Error: {1}".format(subnet['name'], e))
                continue
            try:
                subnet_id = self.fm.subnets.show(subnet['name'])['id']
                log.log(log.LOG_DEBUG, "Subnet '{0}' (id={1}) already present.".format(subnet['name'], subnet_id))
            except:
                # get domain_ids
                add_domain = []
                for subnet_domain in subnet['domain']:
                    try:
                        dom_id = self.fm.domains.show(subnet_domain['name'])['id']
                        add_domain.append(dom_id)
                    except:
                        log.log(log.LOG_WARN, "Cannot get ID of Domain '{0}', skipping".format(subnet_domain['name']))

                # get dhcp_proxy_id
                dhcp_proxy_id = False
                try:
                    dhcp_proxy_id = self.fm.smart_proxies.show(subnet['dhcp-proxy'])['id']
                except:
                    log.log(log.LOG_WARN, "Cannot get ID of DHCP Smart Proxy '{0}', skipping".format(subnet['dhcp-proxy']))

                # get tftp_proxy_id
                tftp_proxy_id = False
                try:
                    tftp_proxy_id = self.fm.smart_proxies.show(subnet['tftp-proxy'])['id']
                except:
                    log.log(log.LOG_WARN, "Cannot get ID of TFTP Smart Proxy '{0}', skipping".format(subnet['tftp-proxy']))

                # get dns_proxy_id
                dns_proxy_id = False
                try:
                    dns_proxy_id = self.fm.smart_proxies.show(subnet['dns-proxy'])['id']
                except:
                    log.log(log.LOG_WARN, "Cannot get ID of DNS Smart Proxy '{0}', skipping".format(subnet['dns-proxy']))

                log.log(log.LOG_INFO, "Create Subnet '{0}'".format(subnet['name']))
                subnet_tpl = {
                    'name':             subnet['name'],
                    'network':          subnet['network'],
                    'mask':             subnet['mask'],
                    'gateway':          subnet['gateway'],
                    'dns_primary':      subnet['dns-primary'],
                    'dns_secondary':    subnet['dns-secondary'],
                    'ipam':             subnet['ipam'],
                    'from':             subnet['from'],
                    'to':               subnet['to'],
                    'vlanid':           subnet['vlanid'],
                    'boot_mode':        subnet['boot-mode']
                }

                if add_domain: subnet_tpl['domain_ids'] = add_domain
                if dhcp_proxy_id: subnet_tpl['dhcp_id'] = dhcp_proxy_id
                if tftp_proxy_id: subnet_tpl['tftp_id'] = tftp_proxy_id
                if dns_proxy_id: subnet_tpl['dns_id'] = dns_proxy_id

                self.fm.subnets.create(subnet=subnet_tpl)



    def process_config_ptable(self):
        log.log(log.LOG_INFO, "Processing Partition Tables")
        for ptable in self.get_config_section('partition-table'):
            try:
                self.validator.ptable(ptable)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Partition Table '{0}': YAML validation Error: {1}".format(ptable['name'], e))
                continue
            try:
                ptable_id = self.fm.ptables.show(ptable['name'])['id']
                log.log(log.LOG_DEBUG, "Partition Table '{0}' (id={1}) already present.".format(ptable['name'], ptable_id))
            except:
                log.log(log.LOG_INFO, "Create Partition Table '{0}'".format(ptable['name']))
                ptable_tpl = {
                    'name':             ptable['name'],
                    'layout':           ptable['layout'],
                    'snippet':          ptable['snippet'],
                    'audit_comment':    ptable['audit-comment'],
                    'locked':           ptable['locked'],
                    'os_family':        ptable['os-family']
                }
                self.fm.ptables.create( ptable = ptable_tpl )



    def process_config_os(self):
        log.log(log.LOG_INFO, "Processing Operating Systems")
        for operatingsystem in self.get_config_section('os'):
            try:
                self.validator.os(operatingsystem)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Operating System '{0}': YAML validation Error: {1}".format(operatingsystem['name'], e))
                continue
            try:
                os_id = self.fm.operatingsystems.show(operatingsystem['description'])['id']
                log.log(log.LOG_DEBUG, "Operating System '{0}' (id={1}) already present.".format(operatingsystem['name'], os_id))
            except:
                log.log(log.LOG_INFO, "Create Operating System '{0}'".format(operatingsystem['name']))
                os_tpl = {
                    'name':             operatingsystem['name'],
                    'description':      operatingsystem['description'],
                    'major':            operatingsystem['major'],
                    'minor':            operatingsystem['minor'],
                    'family':           operatingsystem['family'],
                    'release_name':     operatingsystem['release-name'],
                    'password_hash':    operatingsystem['password-hash']
                }
                os_obj = self.fm.operatingsystems.create(operatingsystem=os_tpl)

                #  host_params
                if operatingsystem['parameters'] is not None:
                    for name,value in operatingsystem['parameters'].iteritems():
                        p = {
                            'name':     name,
                            'value':    value
                        }
                        try:
                            self.fm.operatingsystems.parameters_create(os_obj['id'], p )
                        except:
                            log.log(log.LOG_WARN, "Error adding host parameter '{0}'".format(name))


    def process_config_provisioningtpl(self):
        log.log(log.LOG_INFO, "Processing Provisioning Templates")
        # fm.provisioning_templates.show(name) does not work as expected, we need to iterate over fm.provisioning_templates.index()
        ptlist = self.fm.provisioning_templates.index(per_page=99999)['results']
        for pt in self.get_config_section('provisioning-template'):
            try:
                self.validator.provt(pt)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Provisioning Template '{0}': YAML validation Error: {1}".format(pt['name'], e))
                continue

            pt_id = False
            for ptc in ptlist:
                if (ptc['name'] == pt['name']):
                    pt_id = ptc['id']
                    log.log(log.LOG_DEBUG, "Provisioning Template '{0}' (id={1}) already present.".format(pt['name'], pt_id))
            if not pt_id:
                log.log(log.LOG_INFO, "Create Provisioning Template '{0}'".format(pt['name']))
                pt_tpl = {
                    'name':             pt['name'],
                    'template':         pt['template'],
                    'snippet':          pt['snippet'],
                    'audit_comment':    pt['audit-comment'],
                    'template_kind_id': pt['template-kind-id'],
                    'locked':           pt['locked']
                }
                os_ids = []
                for osc in pt['os']:
                    try:
                        os_id = self.fm.operatingsystems.show(osc['name'])['id']
                        os_ids.append(os_id)
                    except:
                        log.log(log.LOG_WARN, "Cannot link OS '{0}' to Provisioning Template '{1}'".format(osc['name'],pt['name']))
                pt_tpl = {
                    'name':                 pt['name'],
                    'template':             pt['template'],
                    'snippet':              pt['snippet'],
                    'audit_comment':        pt['audit-comment'],
                    'template_kind_id':     pt['template-kind-id'],
                    'locked':               pt['locked'],
                    'operatingsystem_ids':  os_ids
                }
                prtes = self.fm.provisioning_templates.create(provisioning_template=pt_tpl)



    def process_template_combination_attribute(self):
        ptlist = self.fm.provisioning_templates.index(per_page=99999)['results']
        envlist = self.fm.environments.index(per_page=99999)['results']
        for pt in self.get_config_section('provisioning-template'):

            msg = ""

            pt_id = False
            for ptc in ptlist:
                if (ptc['name'] == pt['name']):
                    pt_id = ptc['id']
            if not pt_id:
                log.log(log.LOG_WARN, "Cannot resolve Provisioning template '{0}' ".format(pt['name']) )
                continue

            if 'template_combination_attribute' not in pt or pt['template-combination-attribute'] is None:
                continue
            else:
                linklist = pt['template-combination-attribute']

            for item in linklist:
                env_id = False
                hg_id = False
                for envc in envlist:
                    try:
                        if (item['enviroment'] == envc['name']):
                            env_id = envc['id']
                    except KeyError:
                        env_id = False
                try:
                    hg_id = self.fm.hostgroups.show(item['hostgroup'])['id']
                except:
                    hg_id = False

                if hg_id is not False or env_id is not False:

                    pt_api_arr = {
                        "template_combinations_attributes": [ {} ]
                    }

                    if hg_id is not False:
                        pt_api_arr["template_combinations_attributes"][0]["hostgroup_id"] = hg_id

                    if env_id is not False:
                        pt_api_arr["template_combinations_attributes"][0]["environment_id"] = env_id

                    try:
                        self.fm.provisioning_templates.update(pt_api_arr, pt_id)
                    except ForemanException as e:
                        msg = self.get_api_error_msg(e)
                        log.log(log.LOG_WARN, "Cannot link provisioning template '{0}' api says: '{1}'".format(pt['name'], msg) )
                        continue
                else:
                    log.log(log.LOG_WARN, "Cannot link provisioning template '{0}', at least hostgroup needs to be valid".format(pt['name'], msg) )


    def process_config_os_link(self):
        #log.log(log.LOG_INFO, "Link Operating System Items (Provisioning Templates, Media, Partition Tables)")
        for operatingsystem in self.get_config_section('os'):
            try:
                self.validator.os(operatingsystem)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot update Operating System '{0}': YAML validation Error: {1}".format(operatingsystem['name'], e))
                continue

            os_obj = False
            try:
                os_obj = self.fm.operatingsystems.show(operatingsystem['description'])
            except:
                log.log(log.LOG_WARN, "Cannot get ID of Operating System '{0}', skipping".format(operatingsystem['name']))
            if os_obj:

                # link Partition Tables
                add_pt = []
                for os_ptable in operatingsystem['partition-table']:
                    try:
                        ptable_id = self.fm.ptables.show(os_ptable['name'])['id']
                        add_pt.append({'id': ptable_id})
                    except:
                        log.log(log.LOG_WARN, "Cannot get ID of Partition Table '{0}', skipping".format(os_ptable['name']))

                # link architectures
                add_arch = []
                for os_arch in operatingsystem['architecture']:
                    try:
                        arch_id = self.fm.architectures.show(os_arch['name'])['id']
                        add_arch.append(arch_id)
                    except:
                        log.log(log.LOG_WARN, "Cannot get ID of Architecture '{0}', skipping".format(os_arch['name']))

                # link medium
                add_medium = []
                medialist = self.fm.media.index(per_page=99999)['results']
                for os_media in operatingsystem['medium']:
                    for mediac in medialist:
                        if mediac['name'] == os_media['name']:
                            add_medium.append(mediac['id'])

                # link Provisioning Templates
                add_osdef = []
                add_provt = []
                ptlist = self.fm.provisioning_templates.index(per_page=99999)['results']
                for os_pt in operatingsystem['provisioning-template']:
                    for ptc in ptlist:
                        if ptc['name'] == os_pt['name']:
                            pto = {
                                #'id':                       os_obj['id'],
                                #'config_template_id':       ptc['id'],
                                'template_kind_id':         ptc['template_kind_id'],
                                'provisioning_template_id': ptc['id'],
                            }
                            add_osdef.append(pto)
                            add_provt.append(ptc['id'])

                # now all mapping is done, update os
                update_tpl = {}
                update_osdef = {}
                if add_pt: update_tpl['ptables'] = add_pt
                if add_arch: update_tpl['architecture_ids']         = add_arch
                if add_medium: update_tpl['medium_ids']             = add_medium
                if add_provt:
                    update_tpl['provisioning_template_ids']           = add_provt
                    update_osdef['os_default_templates_attributes']   = add_osdef

                log.log(log.LOG_INFO, "Linking Operating System '{0}' to Provisioning Templates, Media and Partition Tables".format(operatingsystem['description']))

                try:
                    self.fm.operatingsystems.update(os_obj['id'], update_tpl)
                    if add_provt:
                        self.fm.operatingsystems.update(os_obj['id'], update_osdef)
                except:
                    log.log(log.LOG_DEBUG, "An Error Occured when linking Operating System '{0}' (non-fatal)".format(operatingsystem['description']))



    def process_config_hostgroup(self):
        for hostgroup in self.get_config_section('hostgroup'):
            # validate yaml
            try:
                self.validator.hostgroup(hostgroup)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create Hostgroup '{0}': YAML validation Error: {1}".format(hostgroup['name'], e))
                continue

            # check if hostgroup already exists
            try:
                hg_id = self.fm.hostgroups.show(hostgroup['name'])['id']
                log.log(log.LOG_DEBUG, "Hostgroup '{0}' (id={1}) already present.".format(hostgroup['name'], hg_id))
                continue

            # hg is not existent on fm, creating
            except:
                log.log(log.LOG_INFO, "Create Hostgroup '{0}'".format(hostgroup['name']))
                hg_parent = hg_env = hg_os = hg_arch = hg_medium = hg_parttbl = hg_domain = hg_subnet = False

                # find parent hostgroup
                try:
                    hg_parent = self.fm.hostgroups.show(hostgroup['parent'])['id']
                except:
                    log.log(log.LOG_DEBUG, "Cannot get ID of Parent Hostgroup '{0}', skipping".format(hostgroup['parent']))

                # find environment
                envlist = self.fm.environments.index(per_page=99999)['results']
                for envc in envlist:
                    if (hostgroup['environment'] == envc['name']):
                        hg_env = envc['id']
                if not hg_env:
                    log.log(log.LOG_DEBUG, "Cannot get ID of Environment '{0}', skipping".format(hostgroup['environment']))

                # find operatingsystem
                try:
                    hg_os = self.fm.operatingsystems.show(hostgroup['os'])['id']
                except:
                    log.log(log.LOG_DEBUG, "Cannot get ID of Operating System '{0}', skipping".format(hostgroup['os']))

                # find architecture
                try:
                    hg_arch = self.fm.architectures.show(hostgroup['architecture'])['id']
                except:
                    log.log(log.LOG_DEBUG, "Cannot get ID of Architecture '{0}', skipping".format(hostgroup['architecture']))

                # find medium
                medialist = self.fm.media.index(per_page=99999)['results']
                for mediac in medialist:
                    if (mediac['name'] == hostgroup['medium']):
                        hg_medium = mediac['id']
                if not hg_medium:
                    log.log(log.LOG_DEBUG, "Cannot get ID of Medium '{0}', skipping".format(hostgroup['medium']))

                # find partition table
                try:
                    hg_parttbl = self.fm.ptables.show(hostgroup['partition-table'])['id']
                except:
                    log.log(log.LOG_DEBUG, "Cannot get ID of Partition Table '{0}', skipping".format(hostgroup['partition-table']))

                # find domain
                try:
                    hg_domain = self.fm.domains.show(hostgroup['domain'])['id']
                except:
                    log.log(log.LOG_DEBUG, "Cannot get ID of Domain '{0}', skipping".format(hostgroup['partition-table']))

                # find subnet
                try:
                    hg_subnet = self.fm.subnets.show(hostgroup['subnet'])['id']
                except:
                    log.log(log.LOG_DEBUG, "Cannot get ID of Subnet '{0}', skipping".format(hostgroup['subnet']))

                # build array
                hg_arr = {
                    'name':         hostgroup['name']
                }
                if hg_parent:
                    hg_arr['parent_id']           = hg_parent
                if hg_env:
                    hg_arr['environment_id']      = hg_env
                if hg_os:
                    hg_arr['operatingsystem_id']  = hg_os
                if hg_arch:
                    hg_arr['architecture_id']     = hg_arch
                if hg_medium:
                    hg_arr['medium_id']           = hg_medium
                if hg_domain:
                    hg_arr['domain_id']           = hg_domain
                if hg_parttbl:
                    hg_arr['ptable_id']           = hg_parttbl
                if hg_subnet:
                    hg_arr['subnet_id']           = hg_subnet

                # send to foreman-api
                try:
                    hg_api_answer = self.fm.hostgroups.create(hostgroup=hg_arr)
                except ForemanException as e:
                    msg = self.get_api_error_msg(e)
                    log.log(log.LOG_ERROR, "An Error Occured when creating Hostgroup '{0}', api says: '{1}'".format(hostgroup['name'], msg) )
                    continue

                try:
                    created_hg_id = hg_api_answer['id']
                except KeyError:
                    log.log(log.LOG_ERROR, "An Error Occured when creating Hostgroup '{0}'".format(hostgroup['name']))
                    continue


                # hostgroup parameters
                if hostgroup['parameters'] is not None:
                    for param_name, param_val in hostgroup['parameters'].iteritems():
                        param_arr = { "name": param_name, "value": param_val }
                        try:
                            self.fm.hostgroups.parameters_create(param_arr, created_hg_id)
                        except ForemanException as e:
                            log.log(log.LOG_ERROR, "An Error Occured when creating Hostgroup param '{0}'".format(param_name) )
                            continue

                #except:
                #    log.log(log.LOG_ERROR, "An Error Occured when creating Hostgroup '{0}'".format(hostgroup['name']))

    def process_config_host(self):
        for hostc in self.get_config_section('host'):
            hostname = "{0}.{1}".format(hostc['name'], hostc['domain'])
            try:
                host = self.fm.hosts.show(hostname)
                log.log(log.LOG_DEBUG, "Dumping Host Response")
                log.log(log.LOG_DEBUG, host, True)
                host_id = host['id']
                log.log(log.LOG_INFO, "Host '{0}' (id={1}) already present.".format(hostc['name'], host_id))
            except:

                # get domain_id
                try:
                    log.log(log.LOG_DEBUG, "Get Domain")
                    domains = self.fm.domains.show(hostc['domain'])
                    log.log(log.LOG_DEBUG, "Dumping Domain Response")
                    log.log(log.LOG_DEBUG, domains, True)
                    domain_id = domains['id']
                    log.log(log.LOG_INFO, "Domain with name {0} found."
                                          "Domain has ID {1}".format(hostc['domain'], domain_id)
                            )
                except:
                    log.log(log.LOG_ERROR, "Domain {0} does not exist".format(hostc['domain']))
                    continue

                # get environment_id
                log.log(log.LOG_DEBUG, "Get Environment")
                enviroment_id = False
                envlist = self.fm.environments.index(per_page=99999)
                log.log(log.LOG_DEBUG, "Dumping Environment Response")
                log.log(log.LOG_DEBUG, envlist, True)
                for envc in envlist['results']:
                    if (hostc['environment'] == envc['name']):
                        enviroment_id = envc['id']
                if not enviroment_id:
                    log.log(log.LOG_WARN, "Cannot get ID of Environment '{0}', skipping".format(hostc['environment']))
                    continue

                # get architecture_id
                try:
                    log.log(log.LOG_DEBUG, "Get Architecture")
                    architecture = self.fm.architectures.show(hostc['architecture'])
                    log.log(log.LOG_DEBUG, "Dumping Architecture Response")
                    log.log(log.LOG_DEBUG, architecture, True)
                    architecture_id = architecture['id']
                    log.log(log.LOG_INFO, "Architecture with name {0} found."
                                          "Architecture has ID {1}".format(hostc['architecture'], architecture_id))
                except:
                    log.log(log.LOG_ERROR, "Architecture {0} does not exist".format(hostc['architecture']))
                    continue

                # get os_id
                try:
                    log.log(log.LOG_DEBUG, "Get Operatingsystem")
                    os = self.fm.operatingsystems.show(hostc['os'])
                    os_id = os['id']
                    log.log(log.LOG_DEBUG, "Dumping Operatingsystem Response")
                    log.log(log.LOG_DEBUG, os, True)
                    log.log(log.LOG_INFO, "Operatingsystem with name {0} found."
                                          "Operatingsystem has ID {1}".format(hostc['os'], os_id))

                except:
                    log.log(log.LOG_ERROR, "OS {0} does not exist".format(hostc['os']))
                    continue

                # get media_id, show() not working here, manual mapping
                log.log(log.LOG_DEBUG, "Get Media")
                media_id = False
                medias = self.fm.media.index()
                log.log(log.LOG_DEBUG, "Dumping Media Response")
                log.log(log.LOG_DEBUG, medias, True)
                for media in medias['results']:
                    if (media['name'] == hostc['media']):
                        media_id = media['id']
                        log.log(log.LOG_INFO, "Media with name {0} found."
                                              "Media has ID {1}".format(hostc['media'], media_id))
                if not media_id:
                    log.log(log.LOG_ERROR, "Media {0} does not exist".format(hostc['media']))
                    continue

                # get parttable_id
                log.log(log.LOG_DEBUG, "Get Partition")
                try:
                    parttable = self.fm.ptables.show(hostc['partition'])
                    log.log(log.LOG_DEBUG, "Dumping Parttable Response")
                    log.log(log.LOG_DEBUG, parttable, True)
                    parttable_id = parttable['id']
                except:
                    log.log(log.LOG_ERROR, "Partition {0} does not exist".format(hostc['partition']))
                    continue

                # get model_id
                log.log(log.LOG_DEBUG, "Get Model")
                try:
                    model = self.fm.models.show(hostc['model'])
                    log.log(log.LOG_DEBUG, "Dumping Model Response")
                    log.log(log.LOG_DEBUG, model, True)
                    model_id = model['id']
                except:
                    log.log(log.LOG_ERROR, "Model '{0}' does not exist".format(hostc['model']))
                    continue

                # get organization_id
                log.log(log.LOG_DEBUG, "Get Organization")
                try:
                    organization = self.fm.organizations.show(hostc['organization'])
                    log.log(log.LOG_DEBUG, "Dumping Organization Response")
                    log.log(log.LOG_DEBUG, organization, True)
                    if len(organization) > 0:
                        organization_id = organization['id']
                    else:
                        log.log(log.LOG_INFO, "Organization %s not found. Searching organization with name" % hostc['organization'])
                        organization_id = filterbyname(self.fm.organizations.index(per_page=99999), hostc['organization'])
                except:
                    log.log(log.LOG_ERROR, "Organization '{0}' does not exist".format(hostc['organization']))
                    continue

                # get location_id
                log.log(log.LOG_DEBUG, "Get Location")
                try:
                    location = self.fm.locations.show(hostc['location'])
                    log.log(log.LOG_DEBUG, "Dumping Location Response")
                    log.log(log.LOG_DEBUG, organization, True)
                    if len(location) > 0:
                        location_id = location['id']
                    else:
                        log.log(log.LOG_INFO, "Location %s not found. Searching location with name" % hostc['location'])
                        location_id = filterbyname(self.fm.locations.index(per_page=99999), hostc['location'])
                except:
                    log.log(log.LOG_ERROR, "Location '{0}' does not exist".format(hostc['location']))
                    continue

                # build host_params array
                host_params = []
                for name,value in hostc['parameters'].iteritems():
                    p = {
                        'name':     name,
                        'nested':   False,
                        'value':    value
                    }
                    host_params.append(p)

                host_tpl = {
                    'managed':              'true',
                    'build':                'true',
                    'name':                 hostc['name'],
                    #'mac':                  hostc['mac'],
                    'domain_id':            domain_id,
                    'environment_id':       enviroment_id,
                    'architecture_id':      architecture_id,
                    'operatingsystem_id':   os_id,
                    'medium_id':            media_id,
                    'ptable_id':            parttable_id,
                    'model_id':             model_id,
                    'root_pass':            hostc['root-pass'],
                    'organization_id':      organization_id,
                    'location_id':          location_id
                }

                # try to get mac
                try:
                    hmac = hostc['mac']
                except:
                    pass
                if hmac:
                    host_tpl['mac'] = hmac

                # get hostgroup_id
                hg_id = False
                try:
                    hg_id = self.fm.hostgroups.show(hostc['hostgroup'])['id']
                    if hg_id:
                        host_tpl['hostgroup_id'] = hg_id
                except:
                    try:
                        log.log(log.LOG_ERROR, "Hostgroup {0} does not exist".format(hostc['hostgroup']))
                        continue
                    except:
                        pass

                # create host & fix interfaces
                log.log(log.LOG_INFO, "Create Host '{0}'".format(hostname))

                fmh = self.fm.hosts.create( host = host_tpl )
                try:
                    fixif = {
                        'id':           fmh['interfaces'][0]['id'],
                        'managed':      'false',
                        'primary':      'true',
                        'provision':    'true'
                    }
                    fixhost = {
                        'interfaces_attributes':        [ fixif ],
                        'host_parameters_attributes':   host_params
                    }
                    try:
                        self.fm.hosts.update(fixhost, fmh['id'])
                        return fmh
                    except:
                        log.log(log.LOG_DEBUG, "An Error Occured when linking Host '{0}' (non-fatal)".format(hostc['name']))
                except:
                    pass

    # roles
    def process_roles(self):
        log.log(log.LOG_INFO, "Processing roles")
        for role in self.get_config_section('roles'):
            # validate yaml
            try:
                self.validator.role(role)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create role '{0}': YAML validation Error: {1}".format(role['name'], e))
                continue

            try:
                rl_id   = self.fm.roles.show(role['name'])['id']
                log.log(log.LOG_WARN, "Role {0} allready exists".format(role['name']))
                continue
            except TypeError:
                pass

            log.log(log.LOG_INFO, "Creating role {0}".format(role['name']))
            apiobj = {
                "name": role["name"]
            }
            try:
                rapi_obj = self.fm.roles.create( role=apiobj )
                created_role_id = rapi_obj['id']
            except:
                log.log(log.LOG_ERROR, "Something went wrong creating role {0}".format(role['name']))
                continue

            # link permissions
            try:
                wanted_perms = role['permissions']
            except KeyError:
                continue

            # since api cannot resolve by name, we need to fetch the whole list here
            try:
                permlist = self.fm.permissions.index(per_page=999999)['results']
            except:
                log.log(log.LOG_ERROR, "Failed to fetch permission list from foreman")
                continue

            for name, permclass in wanted_perms.iteritems():
                permclass_ids = []
                for perm in permclass:
                    for pcand in permlist:
                        if pcand['name']==perm:
                            permclass_ids.append(pcand['id'])
                # resolve ids was successfull, try to add the filters:
                if len(permclass_ids)>0:
                    log.log(log.LOG_INFO, "Adding permission group {0} to role {1}".format(name, role["name"]))
                    fapiobj = {
                        "role_id": created_role_id,
                        "permission_ids": permclass_ids
                    }
                    try:
                        self.fm.filters.create(filter=fapiobj)
                    except ForemanException as e:
                        msg = self.get_api_error_msg(e)
                        log.log(log.LOG_ERROR, "Cannot link permission group '{0}', api says: '{1}'".format(name, msg) )
                        continue
                # else print a warning
                else:
                    log.log(log.LOG_WARN,"Failed to resolve any permissions of group {0}, skipping".format(name))

            continue



    # ldap auth sources
    def process_auth_sources_ldap(self):
        log.log(log.LOG_INFO, "Processing LDAP auth sources")
        for auth in self.get_config_section('auth-source-ldap'):
            # validate yaml
            try:
                self.validator.auth_source_ldaps(auth)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create LDAP source '{0}': YAML validation Error: {1}".format(auth['name'], e))
                continue
            try:
                as_id   = self.fm.auth_source_ldaps.show(auth['name'])['id']
                log.log(log.LOG_WARN, "LDAP source {0} allready exists".format(auth['name']))
                continue
            except TypeError:
                pass
            ldap_auth_obj = self.dict_underscore(auth)
            try:
                self.fm.auth_source_ldaps.create( auth_source_ldap=ldap_auth_obj )
            except:
                log.log(log.LOG_ERROR, "Something went wrong creating LDAP source {0}".format(auth['name']))



    def process_usergroups(self):
        log.log(log.LOG_INFO, "Processing usergroups")
        for group in self.get_config_section('usergroups'):
            # validate yaml
            try:
                self.validator.usergroup(group)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create usergroup '{0}': YAML validation Error: {1}".format(group['name'], e))
                continue
            try:
                as_id   = self.fm.usergroups.show(group['name'])['id']
                #pprint(dir(self.fm.usergroups))
                log.log(log.LOG_WARN, "Usergroup source {0} allready exists".format(group['name']))
                continue
            except TypeError:
                pass

            # find users
            user_ids = []
            if group['users'] is not None:
                try:
                    for user in group['users']:
                        find = self.fm.users.show(user['name'])
                        try:
                            user_ids.append(find['id'])
                        except TypeError:
                            pass
                except KeyError:
                    pass

            # find groups
            group_ids = []
            if group['groups'] is not None:
                try:
                    for sub_group in group['groups']:
                        find = self.fm.usergroups.show(sub_group['name'])
                        try:
                            group_ids.append(find['id'])
                        except TypeError:
                            pass
                except KeyError:
                    pass

            # find roles
            role_ids = []
            if group['roles'] is not None:
                try:
                    for role in group['roles']:
                        find = self.fm.roles.show(role['name'])
                        try:
                            role_ids.append(find['id'])
                        except TypeError:
                            pass
                except KeyError:
                    pass

            fm_arr = {
                'name': group['name']
            }

            if user_ids:
                fm_arr['user_ids'] = user_ids
            if group_ids:
                fm_arr['usergroup_ids'] = group_ids
            if role_ids:
                fm_arr['role_ids'] = role_ids

            try:
                fm_arr['admin'] = group['admin'];
            except KeyError:
                pass

            pprint(fm_arr)

            try:
                ug_id = self.fm.usergroups.create(usergroup=fm_arr)
            except:
                log.log(log.LOG_ERROR, "Error creating usergroup {0}".format(group['name']))
                continue

            # external usergroups
            if group['ext-usergroups'] is not None:
                try:
                    for auth in group['ext-usergroups']:
                        as_id   = self.fm.auth_source_ldaps.show(auth['auth-source-ldap'])['id']
                        as_obj = {
                            'name':             auth['name'],
                            'auth_source_id':   as_id
                            }
                        try:
                            self.fm.usergroups.external_usergroups_create(group['name'], external_usergroup=as_obj)
                        except TypeError:
                            log.log(log.LOG_ERROR, "Error adding external group {0} to usergroup {1}".format(auth['name'], group['name']))
                except KeyError:
                    continue


    def process_config_user(self):
        log.log(log.LOG_INFO, "Processing users")
        for user in self.get_config_section('users'):
            # validate yaml
            try:
                self.validator.user(user)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot create User '{0}': YAML validation Error: {1}".format(user['login'], e))
                continue
            try:
                as_id   = self.fm.users.show(user['login'])['id']
                log.log(log.LOG_WARN, "User  {0} allready exists".format(user['login']))
                continue
            except TypeError:
                pass

            # resolve auth source
            if user['auth-source'] is not 'INTERNAL':
                try:
                    as_id   = self.fm.auth_source_ldaps.show(user['auth-source'])['id']
                except TypeError:
                    log.log(log.LOG_ERROR, "Cannot resolve auth source '{0}' for user '{1}', skipping creation".format(user['login'], user['auth-source']))
                    continue
                del(user['auth-source'])
                user['auth_source_id'] = as_id
            else:
                del(user['auth-source'])
                user['auth_source_id'] = 1

            try:
                self.fm.users.create(user=user)
            except ForemanException as e:
                msg = self.get_api_error_msg(e)
                log.log(log.LOG_ERROR, "Cannot create user '{0}', api says: '{1}'".format(user['login'], msg) )
                continue



import sys
import logging
import log
from foreman.client import Foreman
from validator import Validator


class ForemanBase:

    def __init__(self, config, loglevel=logging.INFO):
        logging.basicConfig(level=loglevel)
        log.LOGLEVEL = loglevel
        self.config = config['foreman']
        self.loglevel = loglevel
        self.validator = Validator()

    def get_config_section(self, section):
        try:
            cfg = self.config[section]
        except:
            cfg = []
        return cfg

    def connect(self):
        try:
            logging.disable(logging.WARNING)
            self.fm = Foreman(self.config['auth']['url'], (self.config['auth']['user'], self.config['auth']['pass']), api_version=2, use_cache=False, strict_cache=False)
            # this is nescesary for detecting faulty credentials in yaml
            self.fm.architectures.index()
            logging.disable(self.loglevel-1)
        except:
            log.log(log.LOG_ERROR, "Cannot connect to Foreman-API")
            sys.exit(1)

    def get_api_error_msg(self, e):
        dr = e.res.json()
        try:
            msg = dr['error']['message']
        except KeyError:
            msg = dr['error']['full_messages'][0]

        return msg

    def get_host(self, host_id):
        host = self.fm.hosts.show(id=host_id)
        return host

    def remove_host(self, host_id):
        try:
            self.fm.hosts.destroy(id=host_id)
            return True
        except:
            return False

    def dict_underscore(self, d):
        new = {}
        for k, v in d.iteritems():
            if isinstance(v, dict):
                v = self.dict_underscore(v)
            new[k.replace('-', '_')] = v
        return new

    def dict_dash(self, d):
        new = {}
        for k, v in d.iteritems():
            if isinstance(v, dict):
                v = self.dict_dash(v)
            new[k.replace('_', '-')] = v
        return new

    def filter_dump(self, dump, wanted_keys):
        ret = {}
        dd = self.dict_dash(dump)
        for setting in dd:
            if setting in wanted_keys:
                value = dd[setting]
                if value is None or value=='':
                    continue
                ret[setting] = value
        return ret

    def get_audit_ip(self, host_id):
        # this is hacky right now since the audits-api has an bug and does not return any audits whe passing host_id
        # host_audits = self.fm.hosts.audits_index(auditable_id=80)
        #ha = self.fm.hosts.index(auditable_id=81)
        host_audits = []
        host_ip = False
        all_audits = self.fm.audits.index(per_page=99999)['results']

        # get audits of specified host
        for audit in all_audits:
            if ( audit['auditable_type'] == 'Host') and (audit['auditable_id'] == host_id):
                host_audits.append(audit)

        # search for audit type audited_changes['build']
        for audit in host_audits:
            if 'installed_at' in audit['audited_changes']:
                try:
                    ll = len(audit['audited_changes']['installed_at'])
                    host_ip = audit['remote_address']
                    return host_ip
                except:
                    pass

        # nothing found, return False
        return False



#!/usr/bin/python
# -*- coding: utf8 -*-

import log
from base import ForemanBase
from voluptuous import MultipleInvalid


class ForemanCleanup(ForemanBase):

    def process_cleanup_arch(self):
        log.log(log.LOG_INFO, "Processing Cleanup of Architectures")
        for arch in self.get_config_section('cleanup-architecture'):
            try:
                self.validator.cleanup_arch(arch)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot delete Architecture '{0}': YAML validation Error: {1}".format(arch['name'], e))
                continue

            try:
                self.fm.architectures.show(arch['name'])['id']
                log.log(log.LOG_INFO, "Delete Architecture '{0}'".format(arch['name']))

                self.fm.architectures.destroy( arch['name'] )
            except:
                log.log(log.LOG_WARN, "Architecture '{0}' already absent.".format(arch['name']))



    def process_cleanup_computeprfl(self):
        log.log(log.LOG_INFO, "Processing Cleanup of Compute profiles")
        for computeprfl in self.get_config_section('cleanup-compute-profile'):
            try:
                self.validator.cleanup_computeprfl(computeprfl)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot delete Compute profile '{0}': YAML validation Error: {1}".format(computeprfl['name'], e))
                continue

            try:
                self.fm.compute_profiles.show(computeprfl['name'])['id']
                log.log(log.LOG_INFO, "Delete Compute profile '{0}'".format(computeprfl['name']))

                self.fm.compute_profiles.destroy( computeprfl['name'] )
            except:
                log.log(log.LOG_WARN, "Compute profile '{0}' already absent.".format(computeprfl['name']))



    def process_cleanup_medium(self):
        log.log(log.LOG_INFO, "Processing Cleanup of Media")
        medialist = self.fm.media.index(per_page=99999)['results']
        for medium in self.get_config_section('cleanup-medium'):
            try:
                self.validator.cleanup_medium(medium)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot delete Medium '{0}': YAML validation Error: {1}".format(medium['name'], e))
                continue

            medium_deleted = False
            # fm.media.show(name) does not work, we need to iterate over fm.media.index()
            for mediac in medialist:
                if (mediac['name'] == medium['name']):
                    medium_deleted = True
                    log.log(log.LOG_INFO, "Delete Medium '{0}'".format(medium['name']))

                    self.fm.media.destroy( medium['name'] )
                    continue
            if not medium_deleted:
                log.log(log.LOG_WARN, "Medium '{0}' already absent.".format(medium['name']))



    def process_cleanup_ptable(self):
        log.log(log.LOG_INFO, "Processing Cleanup of Partition Tables")
        for ptable in self.get_config_section('cleanup-partition-table'):
            try:
                self.validator.cleanup_ptable(ptable)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot delete Partition Table '{0}': YAML validation Error: {1}".format(ptable['name'], e))
                continue

            try:
                self.fm.ptables.show(ptable['name'])['id']
                log.log(log.LOG_INFO, "Delete Partition Table '{0}'".format(ptable['name']))

                self.fm.ptables.destroy( ptable['name'] )
            except:
                log.log(log.LOG_WARN, "Partition Table '{0}' already absent.".format(ptable['name']))



    def process_cleanup_provisioningtpl(self):
        log.log(log.LOG_INFO, "Processing Cleanup of Provisioning Templates")
        ptlist = self.fm.provisioning_templates.index(per_page=99999)['results']
        for pt in self.get_config_section('cleanup-provisioning-template'):
            try:
                self.validator.cleanup_provt(pt)
            except MultipleInvalid as e:
                log.log(log.LOG_WARN, "Cannot delete Provisioning Template '{0}': YAML validation Error: {1}".format(pt['name'], e))
                continue

            # fm.provisioning_templates.show(name) does not work as expected, we need to iterate over fm.provisioning_templates.index()
            pt_deleted = False
            for ptc in ptlist:
                if (ptc['name'] == pt['name']):
                    pt_deleted = True
                    log.log(log.LOG_INFO, "Delete Provisioning Template '{0}'".format(pt['name']))

                    self.fm.provisioning_templates.destroy( pt['name'] )
                    continue
            if not pt_deleted:
                log.log(log.LOG_WARN, "Provisioning Template '{0}' already absent.".format(pt['name']))



#!/usr/bin/python
# -*- coding: utf8 -*-


import sys
import logging
import log
from base import ForemanBase
import yaml
from collections import OrderedDict
from pprint import pprint
from foreman.client import Foreman, ForemanException


import re
from cStringIO import StringIO

def _fix_dump(dump, indentSize=2):
    stream = StringIO(dump)
    out = StringIO()
    pat = re.compile('(\s*)([^:]*)(:*)')
    last = None

    prefix = 0
    for s in stream:
        indent, key, colon = pat.match(s).groups()
        if indent=="" and key[0]!= '-':
            prefix = 0
        if last:
            if len(last[0])==len(indent) and last[2]==':':
                if all([
                        not last[1].startswith('-'),
                        s.strip().startswith('-')
                        ]):
                    prefix += indentSize
        out.write(" "*prefix+s)
        last = indent, key, colon
    return out.getvalue()



class ForemanDump(ForemanBase):



    # dump functionality
    def dump(self):
        dumpdata = {}
        dumpdata['hosts']       = self.dump_hosts()
        dumpdata['hostgroup']   = self.dump_hostgroups()
        dumpdata['architecture'] = self.dump_arch()
        dumpdata['environment']  = self.dump_env()
        dumpdata['os']           = self.dump_os()
        dumpdata['model']        = self.dump_model()
        dumpdata['media']        = self.dump_media()
        dumpdata['domain']       = self.dump_domain()
        dumpdata['settings']     = self.dump_settings()
        dumpdata['subnet']       = self.dump_subnet()
        dumpdata['smart-proxy']  = self.dump_smartproxy()
        dumpdata['partition-table']  = self.dump_ptable()
        dumpdata['provisioning-template']  = self.dump_provisioningtpl()
        dumpdata['users']        = self.dump_users()
        dumpdata['users']        = self.dump_users()
        dumpdata['auth-source-ldap'] = self.dump_ldaps()
        dumpdata['usergroups'] = self.dump_usergroups()
        dumpdata['roles'] = self.dump_roles()

        # print the result
        fmyml = { 'foreman': dumpdata }

        def str_presenter(dumper, data):
            try:
                dlen = len(data.splitlines())
            except TypeError:
                return dumper.represent_scalar('tag:yaml.org,2002:str', data)
            if (dlen > 1):
                return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')

            return dumper.represent_scalar('tag:yaml.org,2002:str', data)

        yaml.add_representer(unicode, str_presenter)
        yaml.add_representer(str, str_presenter)

        yml = yaml.dump(fmyml, allow_unicode=True, default_flow_style=False )
        print( (yml) )


    def dump_hosts(self):
        ret = []
        all_hosts = self.fm.hosts.index(per_page=99999)['results']
        for host in all_hosts:
            host_tpl = {}
            for setting in host:
                value = host[setting]
                if value is None or value=='':
                    continue
                if (setting == "name"):
                    host_tpl['name'] = value
                elif (setting == "operatingsystem_name"):
                    host_tpl['os'] = value
                elif (setting == "environment_name"):
                    host_tpl['environment'] = value
                elif (setting == "architecture_name"):
                    host_tpl['architecture'] = value
                elif (setting == "medium_name"):
                    host_tpl['media'] = value
                elif (setting == "domain_name"):
                    host_tpl['domain'] = value
                elif (setting == "ptable_name"):
                    host_tpl['partition'] = value
                elif (setting == "model_name"):
                    host_tpl['model'] = value
                elif (setting == "hostgroup_name"):
                    host_tpl['hostgroup'] = value
            # host params
            try:
                hobj = self.fm.hostgroups.show(group['id'])
                if (len(hobj['parameters'])>0):
                    host_tpl['parameters'] = {}
                    for param in hobj['parameters']:
                        host_tpl['parameters'][param['name']] = param['value']
            except:
                pass

            ret.append(host_tpl)
        return ret


    def dump_hostgroups(self):
        ret = []
        all_groups = self.fm.hostgroups.index(per_page=99999)['results']
        for group in all_groups:
            grp_tpl = {}
            for setting in group:
                value = group[setting]
                if value is None or value=='':
                    continue
                if (setting == "name"):
                    grp_tpl['name'] = value
                elif (setting == "operatingsystem_name"):
                    grp_tpl['os'] = value
                elif (setting == "environment_name"):
                    grp_tpl['environment'] = value
                elif (setting == "architecture_name"):
                    grp_tpl['architecture'] = value
                elif (setting == "medium_name"):
                    grp_tpl['media'] = value
                elif (setting == "domain_name"):
                    grp_tpl['domain'] = value
                elif (setting == "ptable_name"):
                    grp_tpl['partition'] = value
                elif (setting == "subnet_name" ):
                    grp_tpl['subnet'] = value
            try:
                hobj = self.fm.hostgroups.show(group['id'])
                if (len(hobj['parameters'])>0):
                    grp_tpl['parameters'] = {}
                    for param in hobj['parameters']:
                        grp_tpl['parameters'][param['name']] = param['value']
            except:
                pass
            ret.append(grp_tpl)
        return ret


    def dump_arch(self):
        ret = []
        all_archs = self.fm.architectures.index(per_page=99999)['results']
        for arch in all_archs:
            ret.append({ 'name': arch['name'] })
        return ret


    def dump_env(self):
        ret = []
        all_envs = self.fm.environments.index(per_page=99999)['results']
        for env in all_envs:
            ret.append({ 'name': env['name'] })
        return ret


    def dump_os(self):
        ret = []
        all_os = self.fm.operatingsystems.index(per_page=99999)['results']
        for os in all_os:
            os_tpl = {}
            for setting in os:
                value = os[setting]
                if value is None or value=='':
                    continue
                if (setting == "name"):
                    os_tpl['name'] = value
                if (setting == "major"):
                    os_tpl['major'] = str(value)
                if (setting == "minor"):
                    os_tpl['minor'] = str(value)
                if (setting == "description"):
                    os_tpl['description'] = value
                if (setting == "release_name"):
                    os_tpl['release_name'] = value
                if (setting == "family"):
                    os_tpl['family'] = value
                if (setting == "password_hash"):
                    os_tpl['password-hash'] = value
            osobj = self.fm.operatingsystems.show(os['id'])
            # media
            if (len(osobj['media'])>0):
                os_tpl['media'] = []
                for media in osobj['media']:
                    os_tpl['media'].append(media['name'])
            # provisioning templates
            if (len(osobj['os_default_templates'])>0):
                os_tpl['provisioning-template'] = []
                for pt in osobj['os_default_templates']:
                    os_tpl['provisioning-template'].append(pt['provisioning_template_name'])
            # architectures
            if (len(osobj['architectures'])>0):
                os_tpl['architectures'] = []
                for pt in osobj['architectures']:
                    os_tpl['architectures'].append(pt['name'])
            # partition tables
            if (len(osobj['ptables'])>0):
                os_tpl['partition-table'] = []
                for pt in osobj['ptables']:
                    os_tpl['partition-table'].append(pt['name'])
            ret.append(os_tpl)
        return ret


    def dump_media(self):
        ret = []
        all_media = self.fm.media.index(per_page=99999)['results']
        mod_tpl = {}
        for medium in all_media:
            med_tpl = {}
            for setting in medium:
                value = medium[setting]
                if value is None or value=='':
                    continue
                if (setting == "name"):
                    med_tpl['name'] = value
                if (setting == "path"):
                    med_tpl['path'] = value
                if (setting == "os_family"):
                    med_tpl['os-family'] = value
            ret.append(med_tpl)
        return ret


    def dump_model(self):
        ret = []
        all_mods = self.fm.models.index(per_page=99999)['results']
        for model in all_mods:
            mod_tpl = {}
            for setting in model:
                value = model[setting]
                if value is None or value=='':
                    continue
                if (setting == "name"):
                    mod_tpl['name'] = value
                if (setting == "hardware_model"):
                    mod_tpl['hardware-model'] = value
                if (setting == "vendor_class"):
                    mod_tpl['vendor-class'] = value
                if (setting == "info"):
                    mod_tpl['info'] = value
            ret.append(mod_tpl)
        return ret


    def dump_domain(self):
        # TODO: dns-proxy
        ret = []
        all_doms = self.fm.domains.index(per_page=99999)['results']
        for dom in all_doms:
            dom_tpl = {}
            for setting in dom:
                value = dom[setting]
                if value is None or value=='':
                    continue
                if (setting == "name"):
                    dom_tpl['name'] = value
                if (setting == "fullname"):
                    dom_tpl['fullname'] = value
                if (setting == "fullname"):
                    dom_tpl['fullname'] = value
            # params
            domobj = self.fm.domains.show(dom['id'])
            if (len(domobj['parameters'])>0):
                dom_tpl['parameters'] = {}
                for param in domobj['parameters']:
                    dom_tpl['parameters'][param['name']] = param['value']
            ret.append(dom_tpl)
        return ret


    def dump_smartproxy(self):
        ret = []
        all_proxys = self.fm.smart_proxies.index(per_page=99999)['results']
        for proxy in all_proxys:
            sp_tpl = {}
            sp_tpl['name'] = proxy['name']
            sp_tpl['url'] = proxy['url']
            ret.append(sp_tpl)
        return ret


    def dump_subnet(self):
        ret = []
        wanted_keys = [
            "name",
            "network",
            "mask",
            "gateway",
            "ipam",
            "from",
            "to",
            "vlanid",
            "dns-primary",
            "dns-secondary",
            "boot-mode"
        ]
        all_subnets = self.fm.subnets.index(per_page=99999)['results']
        for subnet in all_subnets:
            subnet_tpl = {}
            dd = self.dict_dash(subnet)
            for setting in dd:
                if setting in wanted_keys:
                    value = dd[setting]
                    if value is None or value=='':
                        continue
                    subnet_tpl[setting] = value
            # tftp
            try:
                subnet_tpl['tftp-proxy'] = dd['tftp']['name']
            except TypeError:
                pass

            # dhcp
            try:
                subnet_tpl['dhcp-proxy'] = dd['dhcp']['name']
            except TypeError:
                pass

            # domains
            subnet_tpl['domain'] = []

            all_doms = self.fm.subnets.domains_index(dd['id'])
            for dom in all_doms['results']:
                subnet_tpl['domain'].append(dom['name'])

            ret.append(subnet_tpl)

        return ret


    def dump_settings(self):
        ret = []
        all_settings = self.fm.settings.index(per_page=99999)['results']
        for settings in all_settings:
            set_tpl = {}
            for setting in settings:
                value = settings[setting]
                if value is None or value=='':
                    continue
                if (setting == "name"):
                    set_tpl['name'] = value
                if (setting == "value"):
                    set_tpl['value'] = value
            # only in settings: allways print out value
            if set_tpl.get('value') is None:
                set_tpl['value'] = ""
            ret.append(set_tpl)
        return ret



    def dump_ptable(self):
        ret = []
        all_ptables = self.fm.ptables.index(per_page=99999)['results']
        for ptable in all_ptables:
            pt_tpl = {}
            for setting in ptable:
                value = ptable[setting]
                if value is None or value=='':
                    continue
                if (setting == "name"):
                    pt_tpl['name'] = value
                if (setting == "os_family"):
                    pt_tpl['os-family'] = value
                #if (setting == "template"):
                #    pt_tpl['layout'] = value
            # all other values need to be fetched from obj itself
            ptobj = self.fm.ptables.show(ptable['id'])
            try:
                pt_tpl['locked'] = ptobj['locked']
            except KeyError:
                pass
            try:
                pt_tpl['snippet'] = ptobj['snippet']
            except KeyError:
                pass
            try:
                pt_tpl['layout'] = str(ptobj['layout'])
                #print type(pt_tpl['layout'])
            except KeyError:
                pass
            ret.append(pt_tpl)

        return ret



    def dump_provisioningtpl(self):
        ret = []
        wanted_keys = [
            "snippet",
            "name",
            "template-kind-id",
            "locked",
            "audit-comment"
        ]
        all_provt = self.fm.provisioning_templates.index(per_page=99999)['results']

        for provt in all_provt:
            pt_tpl = {}
            dd = self.dict_dash(provt)
            for setting in dd:
                if setting in wanted_keys:
                    value = dd[setting]
                    if value is None or value=='':
                        continue
                    pt_tpl[setting] = value

            pto = self.fm.provisioning_templates.show(dd['id'])
            pt_tpl['template'] = (pto['template'])
            ret.append(pt_tpl)

        return ret


    def dump_users(self):
        ret = []
        wanted_keys = [
            "login",
            "firstname",
            "lastname",
            "locale",
            "mail",
            "timezone"
        ]
        all_users = self.fm.users.index(per_page=99999)['results']
        for user in all_users:
            usr_tpl = {}
            dd = self.dict_dash(user)
            for setting in dd:
                if setting in wanted_keys:
                    value = dd[setting]
                    if value is None or value=='':
                        continue
                    usr_tpl[setting] = value
            auths = dd['auth-source-name']
            if (auths == 'Internal'):
                auths = 'INTERNAL'
            usr_tpl['auth-source'] = auths
            ret.append(usr_tpl)

        return ret


    def dump_ldaps(self):
        ret = []
        wanted_keys = [
            "name",
            "host",
            "port",
            "base-dn",
            "ldap-filter",
            "tls",
            "onthefly-register",
            "usergroup-sync",
            "attr-firstname",
            "attr-lastname",
            "attr-login",
            "attr-mail",
            "attr-photo",
            "mail",
            "timezone"
        ]
        all_ldaps = self.fm.auth_source_ldaps.index(per_page=99999)['results']
        for ldaps in all_ldaps:
            dump_obj = self.filter_dump(ldaps, wanted_keys)
            ret.append(dump_obj)
        return ret


    def dump_usergroups(self):
        ret = []
        all_groups = self.fm.usergroups.index(per_page=99999)['results']
        for group in all_groups:
            gobj = {
                "name": group['name'],
                "admin": group['admin']
            }
            ret.append(gobj)
            # users
            uobj = self.fm.usergroups.users_index(group['id'])['results']
            if (len(uobj)>0):
                gobj['users'] = []
                for user in uobj:
                    add_u = { "name":user['login'] }
                    gobj['users'].append(add_u)
        return ret


    def dump_roles(self):
        ret = []
        all_roles = self.fm.roles.index(per_page=99999)['results']
        for role in all_roles:
            ret.append(
                self.filter_dump(role, ["name"] )
            )
        return ret



# -*- coding: UTF-8 -*-

"""Setuptools package definition"""

from setuptools import setup
from setuptools import find_packages
import os
import codecs


__version__  = None
version_file = "foreman_yml/version.py"
with codecs.open(version_file, encoding="UTF-8") as f:
    code = compile(f.read(), version_file, 'exec')
    exec(code)


def find_data(packages, extensions):
    """Finds data files along with source.

    :param   packages: Look in these packages
    :param extensions: Look for these extensions
    """
    data = {}
    for package in packages:
        package_path = package.replace('.', '/')
        for dirpath, _, filenames in os.walk(package_path):
            for filename in filenames:
                for extension in extensions:
                    if filename.endswith(".%s" % extension):
                        file_path = os.path.join(
                            dirpath,
                            filename
                        )
                        file_path = file_path[len(package) + 1:]
                        if package not in data:
                            data[package] = []
                        data[package].append(file_path)
    return data


with open('README.rst', 'r') as f:
    README_TEXT = f.read()

setup(
    name = "foreman-yml",
    version = __version__,
    packages = find_packages(),
    package_data=find_data(
        find_packages(), ["json", "py"]
    ),
    entry_points = {
        'console_scripts': [
            'foreman-yml = foreman_yml.main:main',
        ]
    },
    install_requires = [
        "pyyaml",
        "requests",
        "python-foreman"
    ],
    author = "Adfinis-SyGroup AG",
    author_email = "info@adfinis-sygroup.ch",
    description = "Foreman YAML client",
    long_description = README_TEXT,
    keywords = "foreman, yaml, api",
    url = "https://github.com/adfinis-sygroup/foreman-yml",
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Topic :: System :: Systems Administration",
    ]
)



import json

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


LOG_DEBUG = 10
LOG_INFO  = 20
LOG_WARN  = 30
LOG_ERROR = 40

global LOGLEVEL
LOGLEVEL = LOG_INFO


def log(level, message, isjson=False):
    global LOGLEVEL
    if (level == LOG_DEBUG):
        pf = "[{0}{1}{2}]".format("", "DEBUG", "")
    if (level == LOG_INFO):
        pf = "[{0}{1}{2}]".format(bcolors.OKGREEN, "INFO", bcolors.ENDC)
    if (level == LOG_WARN):
        pf = "[{0}{1}{2}]".format(bcolors.WARNING, "WARNING", bcolors.ENDC)
    if (level == LOG_ERROR):
        pf = "[{0}{1}{2}]".format(bcolors.FAIL, "ERROR", bcolors.ENDC)
    if level >= LOGLEVEL:
        if not isjson:
            print("{0} {1}".format(pf, message))
        else:
            print("{0} {1}".format(pf, json.dumps(message,
                                                  sort_keys=True,
                                                  indent=2,
                                                  separators=(',', ': ')
                                                  )))



#!/usr/bin/python
# -*- coding: utf8 -*-

import yaml
import sys
from importer import ForemanImport
from dump import ForemanDump
from cleanup import ForemanCleanup
import os
import log


def fm_dump(fm):
    fm.dump()


def fm_cleanup(fm):
    # cleanup architecture
    fm.process_cleanup_arch()

    # cleanup medium
    fm.process_cleanup_medium()

    # cleanup partition table
    fm.process_cleanup_ptable()

    # cleanup provisioning template
    fm.process_cleanup_provisioningtpl()


def fm_import(fm):
    # setting
    fm.process_config_settings()

    # architecture
    fm.process_config_arch()

    # smart proxy
    fm.process_config_smartproxy()

    # domain
    fm.process_config_domain()

    # subnet
    fm.process_config_subnet()

    # environment
    fm.process_config_enviroment()

    # model
    fm.process_config_model()

    # medium
    fm.process_config_medium()

    # partition table
    fm.process_config_ptable()

    # operating system
    fm.process_config_os()

    # provisioning template
    fm.process_config_provisioningtpl()

    # hostgroup
    fm.process_config_hostgroup()

    # Link items to operating system
    fm.process_config_os_link()

    # Link template-combination-attribute
    fm.process_template_combination_attribute()

    # host
    fm.process_config_host()

    # enterprise edition only: ldap
    fm.process_auth_sources_ldap()

    # users
    fm.process_config_user()

    # user groups
    fm.process_usergroups()

    # roles
    fm.process_roles()


def main():

    try:
        function = sys.argv[1]
    except:
        log.log(log.LOG_ERROR, "No action defined (Valid: dump, import, cleanup)")
        sys.exit(1)

    if os.path.isfile(sys.argv[1]):
        config_file = sys.argv[1]
        function = "legacy"
    else:
        try:
            config_file = sys.argv[2]
        except IndexError:
            log.log(log.LOG_ERROR, "No YAML provided")
            sys.exit(1)

    try:
        config_file = open(config_file, 'r')
        config = yaml.load(config_file)
        config_file.close()
    except:
        log.log(log.LOG_ERROR, "Failed to load/parse config")
        sys.exit(1)

    if (function == "import"):
        fm = ForemanImport(config)
        fm.connect()
        fm_import(fm)

    if (function == "dump"):
        fm = ForemanDump(config)
        fm.connect()
        fm_dump(fm)

    if (function == "cleanup"):
        fm = ForemanCleanup(config)
        fm_cleanup(fm)

    if (function == "legacy"):
        fm_cls = ForemanCleanup(config)
        fm_cls.connect()
        fm_cleanup(fm_cls)
        fm_imp = ForemanImport(config)
        fm_imp.connect()
        fm_import(fm_imp)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf8 -*-


from voluptuous import Schema, Required, All, Optional, Any


class Validator:

    def __init__(self):

        self.arch = Schema({
            Required('name'):                           All(str),
        })

        self.domain = Schema({
            Required('name'):                           All(str),
            Optional('fullname'):                       Any(str, None),
            Optional('dns-proxy'):                      Any(str, None),
            Optional('parameters'):                     Any(dict, None)
        })

        self.enviroment = Schema({
            Required('name'):                           All(str)
        })

        self.model = Schema({
            Required('name'):                           All(str),
            Optional('info'):                           Any(str, None),
            Optional('vendor-class'):                   Any(str, None),
            Optional('hardware-model'):                 Any(str, None)
        })

        self.medium = Schema({
            Required('name'):                           All(str),
            Required('path'):                           All(str),
            Required('os-family'):                      All(str),
        })

        self.ptable = Schema({
            Required('name'):                           All(str),
            Required('layout'):                         All(str),
            Optional('snippet'):                        Any(bool,None),
            Optional('os-family'):                      Any(str,None),
            Optional('audit-comment'):                  Any(str,None),
            Optional('locked'):                         Any(bool,None),
        })

        self.provt = Schema({
            Required('name'):                           All(str),
            Required('template'):                       All(str),
            Optional('snippet'):                        Any(bool,None),
            Optional('audit-comment'):                  Any(str,None),
            Optional('template-kind-id'):               Any(int,None),
            Optional('template-combination-attribute'): Any(int, list, dict, None),
            Optional('locked'):                         Any(bool,None),
            Optional('os'):                             Any(None, Schema([{
                Required('name'):                       All(str)
            }]))
        })

        self.os = Schema({
            Required('name'):                           All(str),
            Required('major'):                          Any(str, int),
            Required('minor'):                          Any(str, int),
            Optional('description'):                    Any(str, None),
            Optional('family'):                         Any(str, None),
            Optional('release-name'):                   Any(str, None),
            Optional('parameters'):                     Any(dict, None),
            Optional('password-hash'):                  Any(
                                                            'MD5',
                                                            'SHA256',
                                                            'SHA512',
                                                            'Base64',
                                                            None
                                                            ),
            Optional('architecture'):                   Any(None, Schema([{
                Required('name'):                       All(str)
            }])),
            Optional('provisioning-template'):          Any(None, Schema([{
                Required('name'):                       All(str)
            }])),
            Optional('medium'):                         Any(None, Schema([{
                Required('name'):                       All(str)
            }])),
            Optional('partition-table'):                Any(None, Schema([{
                Required('name'):                       All(str)
            }]))
        })

        self.host = Schema({
            Required('name'):                           All(str),
            Required('template'):                       All(str),
            Optional('snippet'):                        Any(bool, None),
            Optional('audit-comment'):                  Any(str, None),
            Optional('mac'):                            Any(str, None),
            Optional('template-kind-id'):               Any(int, None),
            Optional('template-combination-attribute'): Any(int, None),
            Optional('locked'):                         Any(bool, None),
            Optional('hostgroup'):                      Any(str, None),
            Optional('location'):                       Any(str, None),
            Optional('organization'):                   Any(str, None),
            Optional('parameters'):                     Any(dict, None),
            Optional('os'):                             Any(None, Schema([{
                Required('name'):                       All(str)
            }]))
        })

        self.hostgroup = Schema({
            Required('name'):                           All(str),
            Optional('parent'):                         Any(str, None),
            Optional('environment'):                    Any(str, None),
            Optional('os'):                             Any(str, None),
            Optional('architecture'):                   Any(str, None),
            Optional('medium'):                         Any(str, None),
            Optional('partition-table'):                Any(str, None),
            Optional('domain'):                         Any(str, None),
            Optional('subnet'):                         Any(str, None),
            Optional('parameters'):                     Any(dict, None)
        })

        self.smartproxy = Schema({
            Required('name'):                           All(str),
            Required('url'):                            All(str),
        })

        self.setting = Schema({
            Required('name'):                           All(str),
            Optional('value'):                          Any(list, str, bool,
                                                            int, None),
        })

        self.subnet =  Schema({
            Required('name'):                           All(str),
            Required('network'):                        All(str),
            Required('mask'):                           All(str),
            Optional('gateway'):                        Any(str, None),
            Optional('dns-primary'):                    Any(str, None),
            Optional('dns-secondary'):                  Any(str, None),
            Optional('ipam'):                           Any('DHCP', 'Internal DB', 'None', None),
            Optional('from'):                           Any(str, None),
            Optional('to'):                             Any(str, None),
            Optional('vlanid'):                         Any(str, int, None),
            Optional('domain'):                         Any(None, Schema([{
                Required('name'):                       All(str)
            }])),
            Optional('dhcp-proxy'):                     Any(str, None),
            Optional('tftp-proxy'):                     Any(str, None),
            Optional('dns-proxy'):                      Any(str, None),
            Optional('boot-mode'):                      Any('Static', 'DHCP', None),
        })

        self.cleanup_arch = Schema({
            Required('name'):                           All(str)
        })

        self.cleanup_computeprfl = Schema({
            Required('name'):                           All(str)
        })

        self.cleanup_medium = Schema({
            Required('name'):                           All(str)
        })

        self.cleanup_ptable = Schema({
            Required('name'):                           All(str)
        })

        self.cleanup_provt = Schema({
            Required('name'):                           All(str)
        })

        self.role = Schema({
            Required('name'):                           All(str),
            Optional('permissions'):                    Any(list, dict, None)
        })

        self.usergroup = Schema({
            Required('name'):                           All(str),
            Optional('users'):                          Any(list, None),
            Optional('groups'):                         Any(list, None),
            Optional('ext-usergroups'):                 Any(list, None),
            Optional('roles'):                          Any(list, None),
            Optional('admin'):                          Any(bool, None)
        })

        self.user = Schema({
            Required('login'):                         All(str),
            Required('password'):                      All(str),
            Required('mail'):                          All(str),
            Optional('auth-source'):                   Any(str, None),
            Optional('firstname'):                     Any(str, None),
            Optional('lastname'):                      Any(str, None),
            Optional('admin'):                         Any(bool, None),
            Optional('timezone'):                      Any(str, None),
            Optional('locale'):                        Any(str, None),
        })

        self.auth_source_ldaps = Schema({
            Required('name'):                           All(str),
            Required('host'):                           All(str),
            Optional('port'):                           Any(int, str, None),
            Optional('account'):                        Any(str, None),
            Optional('account-password'):               Any(str, None),
            Optional('base-dn'):                        Any(str, None),
            Optional('attr-login'):                     Any(str, None),
            Optional('attr-firstname'):                 Any(str, None),
            Optional('attr-lastname'):                  Any(str, None),
            Optional('attr-mail'):                      Any(str, None),
            Optional('attr-photo'):                     Any(str, None),
            Optional('onthefly-register'):              Any(bool, int, None),
            Optional('usergroup-sync'):                 Any(bool, int, None),
            Optional('tls'):                            Any(bool, int, None),
            Optional('groups-base'):                    Any(str, None),
            Optional('ldap-filter'):                    Any(str, None),
            Optional('server-type'):                    Any('free_ipa',
                                                            'active_directory',
                                                            'posix',
                                                            None
                                                            )
        })
